package dev.flyfish.framework.compiler.support;

import dev.flyfish.framework.compiler.DynamicJavaCompiler;
import dev.flyfish.framework.compiler.core.VendorJavaCompiler;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;

/**
 * 使用第三方成熟框架
 *
 * @author wangyu
 */
@Slf4j
public class DelegateJavaCompiler implements DynamicJavaCompiler {

    // 第三方编译器
    private final VendorJavaCompiler compiler = VendorJavaCompiler.string();

    private DelegateJavaCompiler() {
    }

    public static DelegateJavaCompiler getInstance() {
        return SingletonHolder.INSTANCE;
    }

    /**
     * 编译java源码，通过字符串
     *
     * @param name   文件名
     * @param source 源码
     * @return 完整类名
     */
    @Override
    public String compile(String name, String source) {
        return compiler.compile(name, source);
    }

    /**
     * 加载类
     * 用于全部编译后，再加载
     *
     * @param name 名称
     * @return 结果
     */
    @Override
    public Class<?> load(String name) {
        try {
            return compiler.getClassLoader().loadClass(name);
        } catch (ClassNotFoundException e) {
            log.warn("类不存在！{}", name);
            return null;
        }
    }

    @Override
    public void close() throws IOException {
        compiler.close();
    }

    private static class SingletonHolder {

        private static final DelegateJavaCompiler INSTANCE = new DelegateJavaCompiler();
    }
}
