package dev.flyfish.framework.query;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.query.support.DomainFunction;

import java.util.List;
import java.util.function.Supplier;

/**
 * lambda查询条件
 *
 * @author wangyu
 */
public interface LambdaQueryChain<T extends Domain> extends QueryChain<LambdaQueryChain<T>, DomainFunction<T>> {

    /**
     * 以且连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    QueryCondition<LambdaQueryChain<T>> and(DomainFunction<T> column);

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> and(Supplier<QueryChain<V, ?>> supplier);

    /**
     * 条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> and(QueryChain<V, ?> chain);

    /**
     * 多个嵌套子条件列表
     *
     * @param chains 多条链
     * @return 结果
     */
    @Override
    default <V extends QueryChain<V, ?>> LambdaQueryChain<T> and(List<QueryChain<V, ?>> chains) {
        return QueryChain.super.and(chains);
    }

    /**
     * 多个嵌套子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> and(List<QueryChain<V, ?>> chains, Queries.Combinator combinator);

    /**
     * 以或连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    QueryCondition<LambdaQueryChain<T>> or(DomainFunction<T> column);

    /**
     * 以或连接条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> or(QueryChain<V, ?> chain);

    /**
     * 以或连接嵌套多个子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> or(List<QueryChain<V, ?>> chains, Queries.Combinator combinator);

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> LambdaQueryChain<T> or(Supplier<QueryChain<V, ?>> supplier);
}
