package dev.flyfish.framework.query;

import java.util.List;
import java.util.function.Supplier;

/**
 * 查询链条
 *
 * @author wangyu
 */
public interface NamedQueryChain extends QueryChain<NamedQueryChain, String> {

    /**
     * 以且连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    QueryCondition<NamedQueryChain> and(String column);

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain and(Supplier<QueryChain<V, ?>> supplier);

    /**
     * 条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain and(QueryChain<V, ?> chain);

    /**
     * 多个嵌套子条件列表
     *
     * @param chains 多条链
     * @return 结果
     */
    @Override
    default <V extends QueryChain<V, ?>> NamedQueryChain and(List<QueryChain<V, ?>> chains) {
        return QueryChain.super.and(chains);
    }

    /**
     * 多个嵌套子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain and(List<QueryChain<V, ?>> chains, Queries.Combinator combinator);

    /**
     * 以或连接下一个字段
     *
     * @param column 列
     * @return 查询条件
     */
    @Override
    QueryCondition<NamedQueryChain> or(String column);

    /**
     * 以或连接条件列表
     *
     * @param chain 多个条件们
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain or(QueryChain<V, ?> chain);

    /**
     * 直接拼接提供者，此处懒加载，最终build才会执行
     *
     * @param supplier 提供者
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain or(Supplier<QueryChain<V, ?>> supplier);

    /**
     * 以或连接嵌套多个子条件列表
     *
     * @param chains     多条链
     * @param combinator 各个链条之间的连接方式
     * @return 结果
     */
    @Override
    <V extends QueryChain<V, ?>> NamedQueryChain or(List<QueryChain<V, ?>> chains, Queries.Combinator combinator);

}
