package dev.flyfish.framework.utils;

import java.nio.charset.StandardCharsets;

/**
 * byte array和hex string互转
 * @author wangyu
 */
public class HexUtils {

    /**
     * 将byte数组转换为表示16进制值的字符串， 如：byte[]{8,18}转换为：0813， 和public static byte[]
     * hexStr2ByteArr(String strIn) 互为可逆的转换过程
     *
     * @param bytes 需要转换的byte数组
     * @return 转换后的字符串
     */
    public static String byteArray2Hex(byte[] bytes) {
        int length = bytes.length;

        // 每个byte用两个字符才能表示，所以字符串的长度是数组长度的两倍
        StringBuilder sb = new StringBuilder(length * 2);
        for (int b : bytes) {
            int temp = b;
            // 把负数转换为正数
            while (temp < 0) {
                temp = temp + 256;
            }
            // 小于0F的数需要在前面补0
            if (temp < 16) {
                sb.append("0");
            }
            sb.append(Integer.toString(temp, 16));
        }
        return sb.toString();
    }

    /**
     * 将表示16进制值的字符串转换为byte数组， 和public static String byteArr2HexStr(byte[] arrB)
     * 互为可逆的转换过程
     *
     * @param hex 需要转换的字符串
     * @return 转换后的byte数组
     */
    public static byte[] hex2ByteArray(String hex) {
        byte[] bytes = hex.getBytes(StandardCharsets.UTF_8);
        int length = bytes.length;

        // 两个字符表示一个字节，所以字节数组长度是字符串长度除以2
        byte[] result = new byte[length / 2];
        for (int i = 0; i < length; i = i + 2) {
            String temp = new String(bytes, i, 2, StandardCharsets.UTF_8);
            result[i / 2] = (byte) Integer.parseInt(temp, 16);
        }
        return result;
    }
}
