package dev.flyfish.framework.configuration.resolver;

import dev.flyfish.framework.configuration.annotations.CurrentUser;
import dev.flyfish.framework.domain.po.User;
import dev.flyfish.framework.utils.UserUtils;
import org.springframework.core.MethodParameter;
import org.springframework.core.ReactiveAdapterRegistry;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.web.reactive.BindingContext;
import org.springframework.web.reactive.result.method.HandlerMethodArgumentResolverSupport;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

/**
 * 用户数据解析器
 *
 * @author wybab
 */
public class UserArgumentResolver extends HandlerMethodArgumentResolverSupport {

    public UserArgumentResolver(ReactiveAdapterRegistry adapterRegistry) {
        super(adapterRegistry);
    }

    @Override
    public boolean supportsParameter(MethodParameter methodParameter) {
        return User.class.isAssignableFrom(methodParameter.getParameterType()) &&
                methodParameter.hasParameterAnnotation(CurrentUser.class);
    }

    @Override
    public Mono<Object> resolveArgument(MethodParameter methodParameter, BindingContext bindingContext, ServerWebExchange serverWebExchange) {
        return ReactiveSecurityContextHolder.getContext()
                .map(UserUtils::extractUser);
    }
}
